<!-- Stylesheet to extract interface names from the Telepathy spec,
based on c-interfaces-generator.xsl.

The output of c-interfaces-generator.xsl has to be altered to be API
compatible with libtelepathy: connection interfaces contain CONN rather than
CONNECTION.

Copyright (C) 2007 Collabora Limited

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:tp="http://telepathy.freedesktop.org/wiki/DbusSpec#extensions-v0"
  exclude-result-prefixes="tp">

  <xsl:import href="glib-interfaces-generator.xsl"/>
  <xsl:output method="text" indent="no" encoding="ascii"/>

  <xsl:variable name="mixed-case-prefix" select="'Tp'"/>
  <xsl:variable name="PREFIX" select="'TP'"/>
  <xsl:variable name="Prefix" select="'Tp'"/>
  <xsl:variable name="prefix" select="'tp'"/>
  <xsl:variable name="upper" select="'ABCDEFGHIJKLMNOPQRSTUVWXYZ'"/>
  <xsl:variable name="lower" select="'abcdefghijklmnopqrstuvwxyz'"/>

  <xsl:template name="mangle-name">
    <xsl:param name="in" select="''"/>
    <xsl:choose>
      <!-- world's least efficient replace() -->
      <xsl:when test="contains($in, 'Connection')">
        <xsl:call-template name="mangle-name">
          <xsl:with-param name="in"
            select="concat(substring-before($in, 'Connection'), 'Conn',
                           substring-after($in, 'Connection'))"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$in"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="interface">
    <xsl:variable name="name">
      <xsl:call-template name="mangle-name">
        <xsl:with-param name="in" select="../@name"/>
      </xsl:call-template>
    </xsl:variable>

    <xsl:if test="../@name != $name">
      <xsl:text>/* Backwards compatibility with libtp &lt; 0.3 */&#10;</xsl:text>
      <xsl:text>#ifndef LIBTELEPATHY_DISABLE_DEPRECATED&#10;</xsl:text>
      <xsl:text>#define TP_IFACE_</xsl:text>
      <xsl:value-of select="translate($name, concat($lower, '/'), $upper)"/>
      <xsl:text> \&#10;TP_IFACE_</xsl:text>
      <xsl:value-of select="translate(../@name, concat($lower, '/'), $upper)"/>
      <xsl:text>&#10;#endif&#10;&#10;</xsl:text>
    </xsl:if>
  </xsl:template>

</xsl:stylesheet>

<!-- vim:set sw=2 sts=2 et noai noci: -->
